import tkinter as tk
from system.gui.utils.grid import grid_configure
from system.gui.widgets.factory import create_widget
from system.gui.modules import ProblemInfo, AdjustableTable, HelpMenu
from system.params import *


class TechnicianLoginView:

    def __init__(self, root):
        self.root = root

        frame_login = create_widget('frame', master=self.root, row=0, column=0)
        create_widget('logo', master=frame_login, row=0, column=0)

        self.widget = {}
        self.widget['ip'] = create_widget('labeled_entry', master=frame_login, row=1, column=0, width=20,
            text='Server IP Address', class_type='string', required=True, required_mark=False)
        self.widget['user'] = create_widget('labeled_entry', master=frame_login, row=2, column=0, width=20,
            text='Username', class_type='string', required=True, required_mark=False)
        self.widget['passwd'] = create_widget('labeled_entry', master=frame_login, row=3, column=0, width=20,
            text='Password', class_type='string', required=False)
        self.widget['task'] = create_widget('labeled_entry', master=frame_login, row=4, column=0, width=20,
            text='Task Name', class_type='string', required=True, required_mark=False)
        self.widget['login'] = create_widget('button', master=frame_login, row=5, column=0, text='Log in')


class TechnicianView:

    def __init__(self, root):
        self.root = root

        self._init_menu()
        self._init_viz()

    def _init_menu(self):
        '''
        Menu initialization
        '''
        self.menu = tk.Menu(master=self.root, relief='raised')
        self.root.config(menu=self.menu)
        self.menu_help = HelpMenu(self.menu)

    def _init_viz(self):
        '''
        '''
        grid_configure(self.root, 0, 0)

        self.widget = {}

        frame_db = create_widget('labeled_frame', master=self.root, row=0, column=0, text='Database')
        self.widget['db_frame'] = frame_db
        self.widget['db_placeholder'] = create_widget('label', master=frame_db, row=0, column=0, text='Uninitialized')
        self.widget['db_table'] = None

        frame_ctrl = create_widget('frame', master=self.root, row=0, column=1, padx=0, pady=0)
        self.widget['problem_info'] = ProblemInfo(master=frame_ctrl, row=0, column=0)

        frame_eval = create_widget('labeled_frame', master=frame_ctrl, row=1, column=0, text='Evaluation')
        self.widget['mode'] = create_widget('radiobutton',
            master=frame_eval, row=0, column=0, text_list=['Manual', 'Auto'], default='Manual')

        frame_manual = create_widget('frame', master=frame_eval, row=1, column=0, padx=0, pady=0)
        self.widget['manual_lock'] = create_widget('button', master=frame_manual, row=0, column=0, text='Lock entry')
        self.widget['manual_release'] = create_widget('button', master=frame_manual, row=0, column=1, text='Release entry')
        self.widget['manual_fill'] = create_widget('button', master=frame_manual, row=1, column=0, columnspan=2, text='Enter performance')

        frame_auto = create_widget('frame', master=frame_eval, row=1, column=0, padx=0, pady=0)
        self.widget['n_worker'] = create_widget('labeled_entry', master=frame_auto, row=0, column=0, columnspan=2, text='Number of workers',
            class_type='int', valid_check=lambda x: x > 0, error_msg='Number of evaluation workers must be positive', required=True, required_mark=False, default=1)
        self.widget['auto_set_program'] = create_widget('button', master=frame_auto, row=1, column=0, text='Set program')
        self.widget['auto_evaluate'] = create_widget('button', master=frame_auto, row=1, column=1, text='Evaluate')

        grid_configure(frame_eval, 0, 0)
        grid_configure(frame_manual, 1, 1)
        grid_configure(frame_auto, 1, 1)
        frame_auto.grid_remove()

        def set_manual():
            frame_auto.grid_remove()
            frame_manual.grid()

        def set_auto():
            frame_manual.grid_remove()
            frame_auto.grid()

        for text, button in self.widget['mode'].widget.items():
            if text == 'Manual':
                button.configure(command=set_manual)
            elif text == 'Auto':
                button.configure(command=set_auto)
            else:
                raise Exception()

        for widget_name in ['auto_set_program', 'auto_evaluate', 'manual_lock', 'manual_release', 'manual_fill']:
            self.widget[widget_name].disable()

    def init_db_table(self, columns):
        '''
        '''
        self.root.geometry(f'{WIDTH}x{HEIGHT}')

        self.widget['db_placeholder'].destroy()
        self.widget['db_table'] = AdjustableTable(master=self.widget['db_frame'], columns=columns)

        for widget_name in ['auto_set_program', 'manual_lock', 'manual_release', 'manual_fill']:
            self.widget[widget_name].enable()

    def get_table_columns(self):
        return self.widget['db_table'].columns